#!/bin/bash

# set sane defaults
IFS=$' \t\n'
export LANG=C
export LC_NUMERIC=C
export LC_TIME=C

die()
{
  printf '%s\n' "$2"
  exit "$1"
}

syntax()
{
  printf 'Usage: [nfile=<path>] [sfile=<path>] [tarball=<path>] '
  printf 'op5 sop pack-anon-ncfg\n'
  exit "$1"
}

msg()
{
  printf '[%s] %s\n' "$(date +%H:%M:%S)" "$1"
}

anon_hash()
{
  printf '%s%s' "$salt" "$1" | openssl sha -sha256 -binary | \
    base64 -w0 | sed 's/[^0-9A-Za-z]//g'
}

anon_oconf()
{
  local data data2 line name regex space1 space2
  regex='^([[:space:]]*)([^[:space:]]+)([[:space:]]+)([^[:space:]]+.*)$'
  while IFS='' read -r line; do
    if [[ $line =~ $regex ]]; then

      space1="${BASH_REMATCH[1]}"
      name="${BASH_REMATCH[2]}"
      space2="${BASH_REMATCH[3]}"
      data="${BASH_REMATCH[4]}"
      data2=''

      case "$name" in
          address|alias|display_name|notes|notes_url|email|pager|address?|_*)
          data2="$(anon_hash "$data")"
          ;;
        check_command|event_handler|host_notification_commands|service_notification_commands)
          [[ $data =~ ^([^!]+)!(.+)$ ]] && \
            data2="${BASH_REMATCH[1]}!$(anon_hash "${BASH_REMATCH[2]}")"
          ;;
        command_line)
          data2="/bin/echo '"'$ARG1$'"'"
          ;;
      esac

      [ -n "$data2" ] && line="$space1$name$space2$data2"
    fi

    printf '%s\n' "$line"
  done
}

anon_sfile()
{
  local data data2 line name regex
  regex='^([^=]+)=(.+)$'
  while IFS='' read -r line; do
    if [[ $line =~ $regex ]]; then

      name="${BASH_REMATCH[1]}"
      data="${BASH_REMATCH[2]}"
      data2=''

      case "$name" in
        long_plugin_output|performance_data|plugin_output|_*|author|comment|comment_data)
          data2="$(anon_hash "$data")"
          ;;
        check_command|event_handler|host_notification_commands|service_notification_commands)
          [[ $data =~ ^([^!]+)!(.+)$ ]] && \
            data2="${BASH_REMATCH[1]}!$(anon_hash "${BASH_REMATCH[2]}")"
          ;;
      esac

      [ -n "$data2" ] && line="$name=$data2"
    fi

    printf '%s\n' "$line"
  done
}

[ "$1" == --help ] && syntax 0
[ "$1" == -h ] && syntax 0
[ -n "$1" ] && syntax 1

[ -n "$nconf" ] || nconf='/opt/monitor/etc/nagios.cfg'
[ -f "$nconf" ] || die '1' "File not found: $nconf"
[ -n "$sfile" ] || sfile='/opt/monitor/var/status.sav'
[ -f "$sfile" ] || die '1' "File not found: $sfile"

if [ -n "$tarball" ]; then
  tarball_path="$(readlink -m -- "$tarball")"
  tarball_name="${tarball_path##*/}"
  [ "$tarball_name" == "${tarball_name%%.*}" ] && \
    die '1' 'No file extension provided for tarball variable.'
  tarball_name="${tarball_name%%.*}"
else
  tarball_name="nconf-anon_${HOSTNAME}_$(date +%Y-%m-%d_%H.%M.%S)"
  tarball_path="$HOME/$tarball_name.tgz"
fi


msg 'Verifying current nagios configuration.'
out="$(/opt/monitor/bin/monitor -v "$nconf")" || die "$?" "$out"

set -u

tmpdir="$(mktemp -d)" || die '1' "Failed creating temp directory."
msg "Created temp directory: $tmpdir"


msg 'Temporarily shutting down Monitor while copying configuration files.'
out="$(mon stop > /dev/null)" || die "$?" "$out"

msg "Copying state retention file: $sfile"
cp -p -- "$sfile" "$tmpdir/status.sav.org" || exit $?

msg "Copying nagios configuration file: $nconf"
grep -v '^cfg_' "$nconf" | \
  egrep -v '^[[:space:]]*(#.*)?$' > "$tmpdir/nagios.cfg"

msg "Copying object configuration files:"
while read -r file; do
  msg "  $file ..."
  egrep -v '^[[:space:]]*(#.*)?$' "$file" >> "$tmpdir/oconf.cfg.org"
done < <(mon oconf files)


msg 'Starting Monitor again.'
out="$(mon start > /dev/null)" || die "$?" "$out"


salt="$(date +%s%N)$RANDOM$RANDOM$RANDOM"

msg "Anonymizing copied state retention data."
anon_sfile < "$tmpdir/status.sav.org" > "$tmpdir/status.sav"

msg "Anonymizing copied object configuration data."
anon_oconf < "$tmpdir/oconf.cfg.org" > "$tmpdir/oconf.cfg"

printf 'cfg_file=oconf.cfg\n' >> "$tmpdir/nagios.cfg"


ts="$(date +%Y-%m-%d_%H.%M.%S)"
name="nconf-anon_${HOSTNAME}_$ts"

msg "Creating archive of anonymized files: $tarball_path"
tar -Pcz --xform "s|^.*/|$tarball_name/|" -f "$tarball_path" \
  "$tmpdir/"{nagios.cfg,oconf.cfg,status.sav}

msg "Finished!"


printf '\nTo view the difference between the original and the anonymized data, '
printf 'use the commands below.\n\n'
printf 'diff -Nup "%s/oconf.cfg"{.org,} | less\n' "$tmpdir"
printf 'diff -Nup "%s/status.sav"{.org,} | less\n' "$tmpdir"

exit 0
